package top.dreamcenter.note;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.*;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;

public class Main {

    private static final int TOTAL_PAGE = 3650;
    private static final String MARK = "start7410";

    private static File file = new File("data/data.txt");

    public static void main(String[] args) {

        JFrame jf = new JFrame("日记本 [F5 上一页; F6 下一页; F7 保存]");

        jf.setSize(800, 1020);
        jf.setResizable(false);
        jf.setLocationRelativeTo(null);
        jf.getContentPane().setLayout(null);
        jf.setBackground(Color.WHITE);
        jf.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        // 内容面板穿透
        JPanel imPanel=(JPanel) jf.getContentPane();
        imPanel.setOpaque(false);

        // 设置背景图片
        URL bkPath = Main.class.getClassLoader().getResource("note.png");
        assert bkPath != null;
        ImageIcon backgroundImage = new ImageIcon(bkPath);
        Image image = backgroundImage.getImage();
        Image newimg = image.getScaledInstance(jf.getWidth() - 50, jf.getHeight(),  java.awt.Image.SCALE_SMOOTH);
        JLabel backgroundLabel = new JLabel(new ImageIcon(newimg));
        backgroundLabel.setBounds(0, 0, jf.getWidth() , jf.getHeight());

        // 数据
        final int[] curPage = {0};
        LinkedList<String> contentlist = new LinkedList<>();
        LinkedList<String> dateList = new LinkedList<>();
        initList(contentlist, dateList);

        // 页号
        JLabel pageNo = new JLabel(String.valueOf(curPage[0]));
        pageNo.setFont(new Font("Segoe Script", Font.PLAIN, 20));
        pageNo.setHorizontalAlignment(JLabel.RIGHT);
        pageNo.setBounds(330,40,60,40);

        // 日期
        JTextField date = new JTextField(dateList.get(curPage[0]));
        date.setBorder(null);
        date.setFont(new Font("Segoe Script", Font.PLAIN, 18));
        date.setBounds(100,45,130,30);

        // 文本行
        JTextArea textArea = new JTextArea(contentlist.get(curPage[0]));
        textArea.setFont(new Font(null, Font.PLAIN, 36));
        textArea.setOpaque(false); // 透明背景
        textArea.setLineWrap(true); // 自动换行
        textArea.setTabSize(2);
        textArea.setBounds(20, 90, jf.getWidth() - 40, jf.getHeight() - 140);

        date.addKeyListener(new KeyAdapter() {
            @Override
            public void keyReleased(KeyEvent e) {
                dateList.set(curPage[0], date.getText());

                switch (e.getKeyCode()) {
                    case KeyEvent.VK_ENTER ->  textArea.requestFocus();
                    case KeyEvent.VK_ESCAPE -> exitSystem(jf, contentlist, dateList);
                }
            }
        });

        textArea.addKeyListener(new KeyAdapter() {
            @Override
            public void keyReleased(KeyEvent e) {
                contentlist.set(curPage[0], textArea.getText());
                switch (e.getKeyCode()) {
                    case KeyEvent.VK_F5 -> { // 上一页
                        if (curPage[0] > 0) {
                            contentlist.set(curPage[0], textArea.getText());
                            dateList.set(curPage[0], date.getText());
                            curPage[0]--;
                            pageNo.setText(String.valueOf(curPage[0]));
                            textArea.setText(contentlist.get(curPage[0]));
                            date.setText(dateList.get(curPage[0]));
                        }
                    }
                    case KeyEvent.VK_F6 -> { // 下一页
                        if (curPage[0] < TOTAL_PAGE) {
                            contentlist.set(curPage[0], textArea.getText());
                            dateList.set(curPage[0], date.getText());
                            curPage[0]++;
                            pageNo.setText(String.valueOf(curPage[0]));
                            textArea.setText(contentlist.get(curPage[0]));
                            date.setText(dateList.get(curPage[0]));
                        }
                    }
                    case KeyEvent.VK_F7 ->  saveData(contentlist, dateList);
                    case KeyEvent.VK_ESCAPE -> exitSystem(jf, contentlist, dateList);
                }
            }
        });

        jf.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                exitSystem(jf, contentlist, dateList);
            }
        });


        // 添加组件
        jf.getLayeredPane().add(backgroundLabel, Integer.valueOf(Integer.MIN_VALUE));
        jf.getContentPane().add(date);
        jf.getContentPane().add(pageNo);
        jf.getContentPane().add(textArea);

        jf.setVisible(true);
    }

    private static void exitSystem(JFrame jf, LinkedList<String> contentList, LinkedList<String> dateList) {
        int confirm = JOptionPane.showConfirmDialog(jf, "是否保存并退出?", "保存退出", JOptionPane.YES_NO_CANCEL_OPTION);
        if (confirm == JOptionPane.YES_OPTION) {
            saveData(contentList, dateList);
            System.exit(0);
        } else if (confirm == JOptionPane.NO_OPTION) {
            System.exit(0);
        }
    }

    private static void initList(List<String> contentList, List<String> dateList) {
        if (!file.exists()) {
            try {
                boolean createResult = file.createNewFile();
                if (!createResult) {
                    JOptionPane.showMessageDialog(null, "数据文件创建失败", "警告", JOptionPane.WARNING_MESSAGE);
                }
            } catch (IOException e) {
                e.printStackTrace();
                JOptionPane.showMessageDialog(null, "IO错误:" + e.getMessage(), "错误", JOptionPane.ERROR_MESSAGE);
            }
        }

        for (int i = 0; i < TOTAL_PAGE; i++){
            contentList.add("");
            dateList.add("");
        }

        try (BufferedReader br = new BufferedReader(
                new InputStreamReader(new FileInputStream(file)))) {
            String tmp;
            StringBuilder sb = new StringBuilder();

            int page = -1;
            String date = "";
            while ((tmp = br.readLine()) != null) {
                if (MARK.equals(tmp)) {
                    if (page != -1) {
                        dateList.set(page - 1, date);
                        contentList.set(page - 1, sb.toString());
                        sb.delete(0, sb.length());
                    }
                    String[] part = br.readLine().split("\\+");
                    page = Integer.parseInt(part[0]);
                    if (part.length == 2){
                        date = part[1];
                    }
                } else {
                    sb.append(tmp).append("\n");
                }
            }
            if (page != -1) {
                dateList.set(page - 1, date);
                contentList.set(page - 1, sb.toString());
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    private static void saveData(List<String> contentList, List<String> dateList){
        try (BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)))) {
            for (int i = 0; i < TOTAL_PAGE; i++) {
                String date = dateList.get(i);
                String content = contentList.get(i);
                if (!"".equals(date) || !"".equals(content)) {
                    bw.write(MARK);
                    bw.newLine();
                    bw.write(String.valueOf(i + 1));
                    bw.write("+");
                    bw.write(date);
                    bw.newLine();
                    bw.write(content);
                    bw.newLine();
                }
            }
            JOptionPane.showMessageDialog(null, "保存成功！", "消息", JOptionPane.INFORMATION_MESSAGE);

        } catch (IOException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(null, "保存失败:" + e.getMessage(), "错误", JOptionPane.ERROR_MESSAGE);
        }
    }
}
